;=============================================================================================================================================
;read_rri_lv1.pro
;Written by: Andrew Howarth and Gareth Perry, University of Calgary
;Created Feburary, 2016
;
;The purpose of this program is to read in an HDF5 level-1 RRI file and return a structure containing data and information from that file.
;There are keywords to return the interpolate ephemeris at a resoultion of 62.5 kHz.
;
;Inputs: 
;  inputFilename - the HDF5 filename to read
;
;Keywords:
;  sec_in - seconds into file (default is 0)
;  total_sec - length of time evaluated from sec_in (default is entire file)
;  decm - decimation factor that reads in one out of every decm data points
;  geographic_interp - option to interpolate geographic coordinates
;  geomag_interp - option to interpolate geomagnetic coordinates
;  gei_gsm_interp - option to interpolate gei and gsm coordinates
;  attitude_interp - option to interpolate yaw, pitch and roll information
;
;Outputs: return a structure, 'rri_dat'
;==============================================================================================================================================



function read_rri_lv1, inputFilename, sec_in=sec_in, total_sec=total_sec, decm=decm, geographic_interp=geographic_interp, $
                       geomag_interp=geomag_interp, gei_gsm_interp=gei_gsm_interp, attitude_interp=attitude_interp
 
  version = 1.4

  print, 'read_rri_lv1, version ' + strtrim(string(version, format='(f0.1)'),2)

  if keyword_set(sec_in) then startpoint=sec_in*62500.33933 else startpoint=0
  
  if keyword_set(total_sec) then endpoint=startpoint+total_sec*62500.33933 else endpoint=!values.f_nan

  if (n_elements(inputFilename) eq 0) then $
    inputFilename = dialog_pickfile(default_extension=".h5", filter="*.h5", /must_exist, title='Open RRI lv1 hdf5 file')
  
  if (n_elements(decm) eq 0) then decm = 1
  
  catch, theError
  if (theError ne 0) then begin
    catch, /cancel
    print, "Error opening " + inputFileName + ". Exiting."
    return,0
  endif else file_id = H5F_OPEN(inputFilename)
  catch, /cancel


;check here to see if the HDF5 data has been read in already or not
;don't run the read-in if they do

common hdf5_data,gain1,gain2,gain3,gain4,switch_k1,sweep_mode_A,sweep_mode_B,bandwidth_A,bandwidth_B,F1_A,F2_A,dwell_A, $
	F1_B,F2_B,dwell_B,df_A,df_B,data_format,chan_A_freq,chan_B_freq,ephemeris_times,gei,vgei,gsm,lat,lon,alt,mlat,mlon, $
	mlt,yaw,pitch,roll,radio_data1,radio_data2,radio_data3,radio_data4,time0,time

common hdf5_file_info,fid


;define a system variable that will help determine whether this is a fresh IDL session or not
defsysv,'!hdf5_dat',exists=gg
if ~gg then begin
defsysv,'!hdf5_dat',0; if this is a fresh session
fid=0
endif

infile=strtrim(strmid(inputfilename,strpos(inputfilename,'RRI_')+4,22),2)

if ~!hdf5_dat or ~strcmp(fid,infile,/fold_case) then begin; if this is the first time running the session, or the user has entered a new file

fid=strtrim(strmid(inputfilename,strpos(inputfilename,'RRI_')+4,22),2)

print,'Reading in HDF5 data....'
   
  ;reading the HDF5 data 
  gain1 = H5_GETDATA(file_id, 'RRI Settings/Antenna 1 Gain')
  gain2 = H5_GETDATA(file_id, 'RRI Settings/Antenna 2 Gain')
  gain3 = H5_GETDATA(file_id, 'RRI Settings/Antenna 3 Gain')
  gain4 = H5_GETDATA(file_id, 'RRI Settings/Antenna 4 Gain')
  switch_k1 = H5_GETDATA(file_id, 'RRI Settings/Antenna Configuration')
  sweep_mode_A = H5_GETDATA(file_id, 'RRI Settings/Sweep Mode A') 
  sweep_mode_B = H5_GETDATA(file_id, 'RRI Settings/Sweep Mode B')
  bandwidth_A = H5_GETDATA(file_id, 'RRI Settings/Bandwidth A (kHz)') 
  bandwidth_B = H5_GETDATA(file_id, 'RRI Settings/Bandwidth B (kHz)')  
  F1_A = H5_GETDATA(file_id, 'RRI Settings/Start Frequency A (Hz)')  
  F2_A = H5_GETDATA(file_id, 'RRI Settings/End Frequency A (Hz)')
  DWELL_A = H5_GETDATA(file_id, 'RRI Settings/Dwell Time A (s)')
  F1_B = H5_GETDATA(file_id, 'RRI Settings/Start Frequency B (Hz)')
  F2_B = H5_GETDATA(file_id, 'RRI Settings/End Frequency B (Hz)')
  DWELL_B = H5_GETDATA(file_id, 'RRI Settings/Dwell Time B (s)')
  DF_A = H5_GETDATA(file_id, 'RRI Settings/Delta Frequency A (Hz)')
  DF_B = H5_GETDATA(file_id, 'RRI Settings/Delta Frequency B (Hz)')
  data_format = H5_GETDATA(file_id, 'RRI Settings/Data Format')
  chan_A_freq = H5_GETDATA(file_id, 'RRI Data/Channel A Frequencies (Hz)')
  chan_B_freq = H5_GETDATA(file_id, 'RRI Data/Channel B Frequencies (Hz)') 
  
  ephemeris_times = H5_GETDATA(file_id, 'CASSIOPE Ephemeris/Ephemeris MET (seconds since May 24, 1968)')
  gei = H5_GETDATA(file_id, 'CASSIOPE Ephemeris/GEI Position (km)')
  vgei = H5_GETDATA(file_id, 'CASSIOPE Ephemeris/GEI Velocity (km per s)')
  gsm = H5_GETDATA(file_id, 'CASSIOPE Ephemeris/GSM Position (km)')  
  lat = H5_GETDATA(file_id, 'CASSIOPE Ephemeris/Geographic Latitude (deg)')
  lon = H5_GETDATA(file_id, 'CASSIOPE Ephemeris/Geographic Longitude (deg)')
  alt = H5_GETDATA(file_id, 'CASSIOPE Ephemeris/Altitude (km)')
  mlat = H5_GETDATA(file_id, 'CASSIOPE Ephemeris/Magnetic Latitude (deg)')
  mlon = H5_GETDATA(file_id, 'CASSIOPE Ephemeris/Magnetic Longitude (deg)')
  mlt = H5_GETDATA(file_id, 'CASSIOPE Ephemeris/MLT (hr)')
  yaw = H5_GETDATA(file_id, 'CASSIOPE Ephemeris/Yaw (deg)')
  pitch = H5_GETDATA(file_id, 'CASSIOPE Ephemeris/Pitch (deg)')
  roll = H5_GETDATA(file_id, 'CASSIOPE Ephemeris/Roll (deg)')
  radio_data1 = H5_GETDATA(file_id, 'RRI Data/Radio Data Monopole 1 (mV)')
  radio_data2 = H5_GETDATA(file_id, 'RRI Data/Radio Data Monopole 2 (mV)')
  radio_data3 = H5_GETDATA(file_id, 'RRI Data/Radio Data Monopole 3 (mV)')
  radio_data4 = H5_GETDATA(file_id, 'RRI Data/Radio Data Monopole 4 (mV)')


  time0 = ephemeris_times[0]

  ;; Time is derived from packet number and time of first packet
  time = indgen(n_elements(radio_data1), /double)/62500.33933 + time0

  !hdf5_dat=1; set the flag high to indicate that this isn't a fresh session

endif; !hdf5_dat option

;check to see if no endpoint was selected, default to last point if true
if ~finite(endpoint) then endpoint=n_elements(radio_data1)-1

;need to interpolate but want to deal with the -180/180 wrap around too. Convert to complex plane.
;this will take too long for a single processor so parallel processing is used
;also,to cut down on computations, we will only interpolate the sample range (startpoint:endpoint) the user asks for
if (endpoint ge n_elements(time)) then endpoint = n_elements(time)-1
if (n_elements(ephemeris_times) lt 3) then begin
  print, 'Not enough ephemeris data provided to interpolate over! Exiting.'
  return, -1
endif

;========== GEOGRAPHIC COORDINATES ==================

if keyword_set(geographic_interp) then begin

  obridge1=obj_new("IDL_IDLBridge",output='')
  obridge2=obj_new("IDL_IDLBridge",output='')
  obridge3=obj_new("IDL_IDLBridge",output='')
  obridge4=obj_new("IDL_IDLBridge",output='')

  obridge1->setvar,'lat',lat
  obridge1->setvar,'ephemeris_times',ephemeris_times
  obridge1->setvar,'time',time
  obridge1->setvar,'decm',decm
  obridge1->setvar,'startpoint',startpoint
  obridge1->setvar,'endpoint',endpoint
  
  obridge2->setvar,'alt',alt
  obridge2->setvar,'ephemeris_times',ephemeris_times
  obridge2->setvar,'time',time
  obridge2->setvar,'decm',decm
  obridge2->setvar,'startpoint',startpoint
  obridge2->setvar,'endpoint',endpoint
 
  obridge3->setvar,'lon',lon
  obridge3->setvar,'ephemeris_times',ephemeris_times
  obridge3->setvar,'time',time
  obridge3->setvar,'decm',decm
  obridge3->setvar,'startpoint',startpoint
  obridge3->setvar,'endpoint',endpoint

  obridge4->setvar,'lon',lon
  obridge4->setvar,'ephemeris_times',ephemeris_times
  obridge4->setvar,'time',time
  obridge4->setvar,'decm',decm
  obridge4->setvar,'startpoint',startpoint
  obridge4->setvar,'endpoint',endpoint

  obridge1->execute,'latitude=spline(ephemeris_times,lat,time[startpoint:endpoint:decm],/double)',/nowait
  obridge2->execute,'altitude=spline(ephemeris_times,alt,time[startpoint:endpoint:decm],/double)',/nowait
  obridge3->execute,'lon_c_i_re=spline(ephemeris_times,cos(lon*!dtor),time[startpoint:endpoint:decm],/double)',/nowait
  obridge4->execute,'lon_c_i_im=spline(ephemeris_times,sin(lon*!dtor),time[startpoint:endpoint:decm],/double)',/nowait
  

  ;ensuring we don't keep exeucting until the parellel processes are done their interpolations
  notdone=1
  while notdone do begin
    if (obridge1->status() eq 0) and (obridge2->status() eq 0) and (obridge3->status() eq 0) and (obridge4->status() eq 0) then notdone=0
  endwhile

  latitude=obridge1->getvar('latitude')
  altitude=obridge2->getvar('altitude')
  lon_c_i_re=obridge3->getvar('lon_c_i_re')
  lon_c_i_im=obridge4->getvar('lon_c_i_im')
  lon_c=complex(lon_c_i_re,lon_c_i_im)
  longitude=atan(imaginary(lon_c),real_part(lon_c))*!radeg
  
  delvar,lon_c_i_re
  delvar,lon_c_i_im
  delvar,lon_c
  
  obj_destroy,obridge1
  obj_destroy,obridge2
  obj_destroy,obridge3
  obj_destroy,obridge4

endif else begin

  latitude=lat[ceil(startpoint/62500.33933):ceil(endpoint/62500.33933)-1]
  longitude=lon[ceil(startpoint/62500.33933):ceil(endpoint/62500.33933)-1]
  altitude=alt[ceil(startpoint/62500.33933):ceil(endpoint/62500.33933)-1]

endelse; geographic_interp keyword

;========== GEOMAGNETIC COORDINATES ==================

if keyword_set(geomag_interp) then begin

  obridge1=obj_new("IDL_IDLBridge",output='')
  obridge2=obj_new("IDL_IDLBridge",output='')
  obridge3=obj_new("IDL_IDLBridge",output='')
  obridge4=obj_new("IDL_IDLBridge",output='')
  obridge5=obj_new("IDL_IDLBridge",output='')
  

  obridge1->setvar,'mlat',mlat
  obridge1->setvar,'ephemeris_times',ephemeris_times
  obridge1->setvar,'time',time
  obridge1->setvar,'decm',decm
  obridge1->setvar,'startpoint',startpoint
  obridge1->setvar,'endpoint',endpoint

  obridge2->setvar,'mlon',mlon
  obridge2->setvar,'time',time
  obridge2->setvar,'ephemeris_times',ephemeris_times
  obridge2->setvar,'decm',decm
  obridge2->setvar,'startpoint',startpoint
  obridge2->setvar,'endpoint',endpoint
  
  obridge3->setvar,'mlon',mlon
  obridge3->setvar,'time',time
  obridge3->setvar,'ephemeris_times',ephemeris_times
  obridge3->setvar,'decm',decm
  obridge3->setvar,'startpoint',startpoint
  obridge3->setvar,'endpoint',endpoint
 
  obridge4->setvar,'mlt',mlt*15.-180.
  obridge4->setvar,'time',time
  obridge4->setvar,'ephemeris_times',ephemeris_times
  obridge4->setvar,'decm',decm
  obridge4->setvar,'startpoint',startpoint
  obridge4->setvar,'endpoint',endpoint
   
  obridge5->setvar,'mlt',mlt*15.-180.
  obridge5->setvar,'time',time
  obridge5->setvar,'ephemeris_times',ephemeris_times
  obridge5->setvar,'decm',decm
  obridge5->setvar,'startpoint',startpoint
  obridge5->setvar,'endpoint',endpoint
  
  obridge1->execute,'maglat=spline(ephemeris_times,mlat,time[startpoint:endpoint:decm],/double)',/nowait
  obridge2->execute,'mlon_c_i_re=spline(ephemeris_times,cos(mlon*!dtor),time[startpoint:endpoint:decm],/double)',/nowait
  obridge3->execute,'mlon_c_i_im=spline(ephemeris_times,sin(mlon*!dtor),time[startpoint:endpoint:decm],/double)',/nowait
  obridge4->execute,'mlt_c_i_re=spline(ephemeris_times,cos(mlt*!dtor),time[startpoint:endpoint:decm],/double)',/nowait
  obridge5->execute,'mlt_c_i_im=spline(ephemeris_times,sin(mlt*!dtor),time[startpoint:endpoint:decm],/double)',/nowait


  ;ensuring we don't keep exeucting until the parellel processes are done their interpolations
  notdone=1
  while notdone do begin
    if (obridge1->status() eq 0) and (obridge2->status() eq 0) and (obridge3->status() eq 0) and (obridge4->status() eq 0) $
	    and (obridge5->status() eq 0) then notdone=0
  endwhile

  maglat=obridge1->getvar('maglat')
  mlon_c_i_re=obridge2->getvar('mlon_c_i_re')
  mlon_c_i_im=obridge3->getvar('mlon_c_i_im')
  mlon_c=complex(mlon_c_i_re,mlon_c_i_im)
  maglon=atan(imaginary(mlon_c),real_part(mlon_c))*!radeg
 
  mlt_c_i_re=obridge4->getvar('mlt_c_i_re')
  mlt_c_i_im=obridge5->getvar('mlt_c_i_im')
  mlt_c=complex(mlt_c_i_re,mlt_c_i_im)
  maglt=(atan(imaginary(mlt_c),real_part(mlt_c))*!radeg+180.)/15.
 
  delvar,mlon_c_i_re
  delvar,mlon_c_i_im
  delvar,mlon_c
  delvar,mlt_c_i_re
  delvar,mlt_c_i_im
  delvar,mlt_c

  ;shutting down the parallel streams now that we're done with them
  obj_destroy,obridge1
  obj_destroy,obridge2
  obj_destroy,obridge3
  obj_destroy,obridge4
  obj_destroy,obridge5

endif else begin

  maglat = mlat[ceil(startpoint/62500.33933):ceil(endpoint/62500.33933)-1]
  maglon = mlon[ceil(startpoint/62500.33933):ceil(endpoint/62500.33933)-1]
  maglt = mlt[ceil(startpoint/62500.33933):ceil(endpoint/62500.33933)-1]

endelse; geomag keyword


  ;========== GEI-BASED COORDINATES ==================
  
if keyword_set(gei_gsm_interp) then begin 

  gsm_interp = make_array(n_elements(time), 3)
  gei_interp = make_array(n_elements(time), 3)
  vgei_interp = make_array(n_elements(time), 3)
  
  obridge1=obj_new("IDL_IDLBridge",output='')
  obridge2=obj_new("IDL_IDLBridge",output='')
  obridge3=obj_new("IDL_IDLBridge",output='')

  obridge1->setvar,'ephemeris_times',ephemeris_times
  obridge2->setvar,'ephemeris_times',ephemeris_times
  obridge3->setvar,'ephemeris_times',ephemeris_times

  obridge1->setvar,'gsm',gsm
  obridge2->setvar,'vgei',vgei
  obridge3->setvar,'gei',gei
 
  obridge1->setvar,'gsm_interp',gsm_interp
  obridge2->setvar,'vgei_interp',vgei_interp
  obridge3->setvar,'gei_interp',gei_interp
 
  obridge1->setvar,'time',time
  obridge2->setvar,'time',time
  obridge3->setvar,'time',time

  obridge1->setvar,'decm',decm
  obridge2->setvar,'decm',decm
  obridge3->setvar,'decm',decm
  
  obridge1->setvar,'startpoint',startpoint
  obridge2->setvar,'startpoint',startpoint
  obridge3->setvar,'startpoint',startpoint
  
  obridge1->setvar,'endpoint',endpoint
  obridge2->setvar,'endpoint',endpoint
  obridge3->setvar,'endpoint',endpoint
  
  obridge1->execute,'for ii = 0, 3-1 do gsm_interp[*, ii] =spline(ephemeris_times,gsm[*,ii],time[startpoint:endpoint:decm],/double)',/nowait
  obridge2->execute,'for ii = 0, 3-1 do vgei_interp[*, ii] =spline(ephemeris_times,vgei[*,ii],time[startpoint:endpoint:decm],/double)',/nowait
  obridge3->execute,'for ii = 0, 3-1 do gei_interp[*, ii] =spline(ephemeris_times,gei[*,ii],time[startpoint:endpoint:decm],/double)',/nowait

  ;ensuring we don't keep exeucting until the parellel processes are done their interpolations
  notdone=1
  while notdone do begin
    if (obridge1->status() eq 0) and (obridge2->status() eq 0) and (obridge3->status() eq 0) then notdone=0
  endwhile

  gsm_interp=obridge1->getvar('gsm_interp')
  vgei_interp=obridge2->getvar('vgei_interp')
  gei_interp=obridge3->getvar('gei_interp')
  
  obj_destroy,obridge1
  obj_destroy,obridge2
  obj_destroy,obridge3
  
endif else begin

  gsm_interp=gsm[ceil(startpoint/62500.33933):ceil(endpoint/62500.33933)-1,*]
  vgei_interp=vgei[ceil(startpoint/62500.33933):ceil(endpoint/62500.33933)-1,*]
  gei_interp=gei[ceil(startpoint/62500.33933):ceil(endpoint/62500.33933)-1,*]

endelse; GEI_GSM keyword



  ;========== YAW-PITCH-ROLL ==================

if keyword_set(attitude_interp) then begin


  ;now to interpolate the YPR with a set of parallel calculations

  obridge1=obj_new("IDL_IDLBridge",output='')
  obridge2=obj_new("IDL_IDLBridge",output='')
  obridge3=obj_new("IDL_IDLBridge",output='')
  obridge4=obj_new("IDL_IDLBridge",output='')
  obridge5=obj_new("IDL_IDLBridge",output='')
  obridge6=obj_new("IDL_IDLBridge",output='')

  obridge1->setvar,'ephemeris_times',ephemeris_times
  obridge2->setvar,'ephemeris_times',ephemeris_times
  obridge3->setvar,'ephemeris_times',ephemeris_times
  obridge4->setvar,'ephemeris_times',ephemeris_times
  obridge5->setvar,'ephemeris_times',ephemeris_times
  obridge6->setvar,'ephemeris_times',ephemeris_times
  
  obridge1->setvar,'time',time
  obridge2->setvar,'time',time
  obridge3->setvar,'time',time
  obridge4->setvar,'time',time
  obridge5->setvar,'time',time
  obridge6->setvar,'time',time

  obridge1->setvar,'yaw',yaw
  obridge2->setvar,'yaw',yaw
  obridge3->setvar,'roll',roll
  obridge4->setvar,'roll',roll
  obridge5->setvar,'pitch',pitch
  obridge6->setvar,'pitch',pitch

  obridge1->setvar,'decm',decm
  obridge2->setvar,'decm',decm
  obridge3->setvar,'decm',decm
  obridge4->setvar,'decm',decm
  obridge5->setvar,'decm',decm
  obridge6->setvar,'decm',decm
  
  obridge1->setvar,'startpoint',startpoint
  obridge2->setvar,'startpoint',startpoint
  obridge3->setvar,'startpoint',startpoint
  obridge4->setvar,'startpoint',startpoint
  obridge5->setvar,'startpoint',startpoint
  obridge6->setvar,'startpoint',startpoint
  
  obridge1->setvar,'endpoint',endpoint
  obridge2->setvar,'endpoint',endpoint
  obridge3->setvar,'endpoint',endpoint
  obridge4->setvar,'endpoint',endpoint
  obridge5->setvar,'endpoint',endpoint
  obridge6->setvar,'endpoint',endpoint
  
  obridge1->execute,'yaw1_re=spline(ephemeris_times,cos(yaw*!dtor),time[startpoint:endpoint:decm],/double)',/nowait
  obridge2->execute,'yaw1_im=spline(ephemeris_times,sin(yaw*!dtor),time[startpoint:endpoint:decm],/double)',/nowait
  obridge3->execute,'roll_re=spline(ephemeris_times,cos(roll*!dtor),time[startpoint:endpoint:decm],/double)',/nowait
 
  ;ensuring we don't keep exeucting until the parellel processes are done their interpolations
  notdone=1
  while notdone do begin
    if (obridge1->status() eq 0) and (obridge2->status() eq 0) and (obridge3->status() eq 0) then notdone=0
  endwhile

  obridge4->execute,'roll_im=spline(ephemeris_times,sin(roll*!dtor),time[startpoint:endpoint:decm],/double)',/nowait
  obridge5->execute,'pitch_re=spline(ephemeris_times,cos(pitch*!dtor),time[startpoint:endpoint:decm],/double)',/nowait
  obridge6->execute,'pitch_im=spline(ephemeris_times,sin(pitch*!dtor),time[startpoint:endpoint:decm],/double)',/nowait

  ;ensuring we don't keep exeucting until the parellel processes are done their interpolations
  notdone=1
  while notdone do begin
    if (obridge4->status() eq 0) and (obridge5->status() eq 0) and (obridge6->status() eq 0) then notdone=0
  endwhile

  yaw_c=complex(obridge1->getvar('yaw1_re'),obridge2->getvar('yaw1_im'))
  roll_c=complex(obridge3->getvar('roll_re'),obridge4->getvar('roll_im'))
  pitch_c=complex(obridge5->getvar('pitch_re'),obridge6->getvar('pitch_im'))

  yaw1=atan(imaginary(yaw_c),real_part(yaw_c))*!radeg
  pitch1=atan(imaginary(pitch_c),real_part(pitch_c))*!radeg
  roll1=atan(imaginary(roll_c),real_part(roll_c))*!radeg

  delvar,yaw_c
  delvar,roll_c
  delvar,pitch_c

  obj_destroy,obridge1
  obj_destroy,obridge2
  obj_destroy,obridge3
  obj_destroy,obridge4
  obj_destroy,obridge5
  obj_destroy,obridge6

endif else begin
	
  yaw1=yaw[ceil(startpoint/62500.33933):ceil(endpoint/62500.33933)-1]
  roll1=roll[ceil(startpoint/62500.33933):ceil(endpoint/62500.33933)-1]
  pitch1=pitch[ceil(startpoint/62500.33933):ceil(endpoint/62500.33933)-1]
	
endelse; attitude keyword

  time_out=time[startpoint:endpoint:decm];narrow and decimate the time too

  
  ;channel frequency arrays dependent if we're a fixed freuqency sweep or not
  if ~strcmp(sweep_mode_A,'Fixed Frequency') then begin
  chan_a_freq_out=chan_a_freq[startpoint:endpoint:decm] 
  chan_b_freq_out=chan_b_freq[startpoint:endpoint:decm] 
  endif else begin
  chan_a_freq_out=make_array(n_elements(radio_data1[startpoint:endpoint:decm]),value=chan_a_freq[0])
  chan_b_freq_out=make_array(n_elements(radio_data1[startpoint:endpoint:decm]),value=chan_a_freq[0])
  endelse

  radio_data1_out=radio_data1[startpoint:endpoint:decm]
  radio_data2_out=radio_data2[startpoint:endpoint:decm]
  radio_data3_out=radio_data3[startpoint:endpoint:decm]
  radio_data4_out=radio_data4[startpoint:endpoint:decm]


  H5F_CLOSE, file_id


  rri_dat={gain1:gain1, gain2:gain2, gain3:gain3, gain4:gain4, switch_k1:switch_k1, sweep_mode_A:sweep_mode_A, sweep_mode_B:sweep_mode_B, $
    bandwidth_A:bandwidth_A, bandwidth_B:bandwidth_B, F1_A:F1_A, F2_A:F2_A, DF_A:DF_A, DWELL_A:DWELL_A, F1_B:F1_B, F2_B:F2_B,$
    DF_B:DF_B, DWELL_B:DWELL_B,data_format:data_format, chan_A_freq:chan_A_freq_out, chan_B_freq:chan_B_freq_out,$
    gei:gei_interp, vgei:vgei_interp, gsm:gsm_interp, lat:latitude, lon:longitude,$
    alt:altitude, mlat:maglat, mlon:maglon, mlt:maglt, yaw:yaw1, $
    pitch:pitch1, roll:roll1, radio_data1:radio_data1_out, radio_data2:radio_data2_out, radio_data3:radio_data3_out, $
    radio_data4:radio_data4_out, time:time_out, no_interp_ephemeris_time:ephemeris_times}

    
return, rri_dat

end
